@@ -6,7 +6,7 @@ use pallas_codec::minicbor::{bytes::ByteVec, data::Int, data::Tag, Decode, Encod
 use pallas_crypto::hash::Hash;
 use std::ops::Deref;
 
-use pallas_codec::utils::{KeyValuePairs, MaybeIndefArray};
+use pallas_codec::utils::{AnyUInt, KeyValuePairs, MaybeIndefArray};
 
 // required for derive attrs to work
 use pallas_codec::minicbor;
@@ -114,12 +114,12 @@ pub type Multiasset<A> = KeyValuePairs<PolicyId, KeyValuePairs<AssetName, A>>;
 
 pub type Mint = Multiasset<i64>;
 
-pub type Coin = u64;
+pub type Coin = AnyUInt;
 
 #[derive(Debug, PartialEq, Clone)]
 pub enum Value {
     Coin(Coin),
-    Multiasset(Coin, Multiasset<u64>),
+    Multiasset(Coin, Multiasset<Coin>),
 }
 
 impl<'b> minicbor::decode::Decode<'b> for Value {
@@ -129,7 +129,7 @@ impl<'b> minicbor::decode::Decode<'b> for Value {
             minicbor::data::Type::U64 => Ok(Value::Coin(d.decode()?)),
             minicbor::data::Type::Array => {
                 d.array()?;
-                let coin = d.u64()?;
+                let coin = d.decode()?;
                 let multiasset = d.decode()?;
                 Ok(Value::Multiasset(coin, multiasset))
             }
@@ -170,7 +170,7 @@ pub struct TransactionOutput {
     pub amount: Value,
 
     #[n(2)]
-    pub datum_hash: Option<ByteVec>,
+    pub datum_hash: Option<Hash<32>>,
 }
 
 pub type PoolKeyhash = Hash<28>;
@@ -699,6 +699,7 @@ pub enum TransactionBodyComponent {
     RequiredSigners(MaybeIndefArray<AddrKeyhash>),
     NetworkId(NetworkId),
 }
+
 impl<'b> minicbor::decode::Decode<'b> for TransactionBodyComponent {
     fn decode(d: &mut minicbor::Decoder<'b>) -> Result<Self, minicbor::decode::Error> {
         let key: u32 = d.decode()?;
@@ -907,7 +908,9 @@ impl minicbor::encode::Encode for NativeScript {
     }
 }
 
-pub type PlutusScript = ByteVec;
+#[derive(Encode, Decode, Debug, PartialEq)]
+#[cbor(transparent)]
+pub struct PlutusScript(#[n(0)] ByteVec);
 
 /*
 big_int = int / big_uint / big_nint ; New
@@ -1057,8 +1060,8 @@ impl minicbor::encode::Encode for PlutusData {
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Constr<A> {
     pub tag: u64,
-    pub prefix: Option<u32>,
-    pub values: MaybeIndefArray<A>,
+    pub any_constructor: Option<u64>,
+    pub fields: MaybeIndefArray<A>,
 }
 
 impl<'b, A> minicbor::decode::Decode<'b> for Constr<A>
@@ -1072,17 +1075,16 @@ where
             Tag::Unassigned(x) => match x {
                 121..=127 | 1280..=1400 => Ok(Constr {
                     tag: x,
-                    values: d.decode()?,
-                    prefix: None,
+                    fields: d.decode()?,
+                    any_constructor: None,
                 }),
                 102 => {
                     d.array()?;
-                    let prefix = Some(d.decode()?);
-                    let values = d.decode()?;
+
                     Ok(Constr {
-                        tag: 102,
-                        prefix,
-                        values,
+                        tag: x,
+                        any_constructor: Some(d.decode()?),
+                        fields: d.decode()?,
                     })
                 }
                 _ => Err(minicbor::decode::Error::message(
@@ -1109,13 +1111,13 @@ where
         match self.tag {
             102 => {
                 e.array(2)?;
-                e.encode(self.prefix)?;
-                e.encode(&self.values)?;
+                e.encode(self.any_constructor.unwrap_or_default())?;
+                e.encode(&self.fields)?;
 
                 Ok(())
             }
             _ => {
-                e.encode(&self.values)?;
+                e.encode(&self.fields)?;
 
                 Ok(())
             }
@@ -1229,7 +1231,7 @@ pub enum Metadatum {
     Bytes(ByteVec),
     Text(String),
     Array(MaybeIndefArray<Metadatum>),
-    Map(Metadata),
+    Map(KeyValuePairs<Metadatum, Metadatum>),
 }
 
 impl<'b> minicbor::Decode<'b> for Metadatum {
@@ -1309,7 +1311,9 @@ impl minicbor::Encode for Metadatum {
     }
 }
 
-pub type Metadata = KeyValuePairs<Metadatum, Metadatum>;
+pub type MetadatumLabel = AnyUInt;
+
+pub type Metadata = KeyValuePairs<MetadatumLabel, Metadatum>;
 
 #[derive(Debug, PartialEq)]
 pub enum AuxiliaryData {
@@ -1442,6 +1446,8 @@ mod tests {
             include_str!("test_data/test19.block"),
             // peculiar block with very BigInt in plutus code
             include_str!("test_data/test20.block"),
+            // peculiar block with bad tx hash
+            include_str!("test_data/test21.block"),
         ];
 
         for (idx, block_str) in test_blocks.iter().enumerate() {
